from django.views.generic.base import View
from django.http.response import HttpResponseBadRequest, HttpResponseRedirect
from . import activate_feature, deactivate_feature


class ActivateFeatureView(View):
    def post(self, request):
        feature = request.POST.get('feature')
        next = request.POST.get('next') or '/'

        if activate_feature(feature, request):
            return HttpResponseRedirect(next)

        return HttpResponseBadRequest('Feature could not be activated.')


class DeactivateFeatureView(View):
    def post(self, request):
        feature = request.POST.get('feature')
        next = request.POST.get('next') or '/'

        if deactivate_feature(feature, request):
            return HttpResponseRedirect(next)

        return HttpResponseBadRequest('Feature could not be deactivated.')



from django.core.cache import cache
from .manager import FeatureList
from .base import FeatureBase, functional_handler_factory
from .signals import (
    feature_activated as fa_signal,
    feature_deactivated as fd_signal
)

import types


__version__ = '1.3.0'


handlers = FeatureList()


__all__ = [
    'handlers',
    'feature_enabled',
    'autodiscover',
    'FeatureBase',
    'register'
]


def feature_enabled(name, request):
    if not getattr(request, 'user', None):
        return False

    def a():
        u = request.user.is_authenticated
        return u() if callable(u) else u

    if a():
        cachekey = 'feature.%s.enabled.%s' % (
            name,
            request.user.username
        )

        if cachekey in cache:
            return cache.get(cachekey)
    else:
        cachekey = None

    if name not in handlers._features:
        return False

    klass, groups = handlers._features.get(name)
    feature = klass(name, groups)
    value = feature.enabled(request)

    if cachekey:
        cache.set(cachekey, value)

    return value


def feature_activated(name, request):
    if not getattr(request, 'user', None):
        return False

    def a():
        u = request.user.is_authenticated
        return u() if callable(u) else u

    if a():
        cachekey = 'feature.%s.activated.%s' % (
            name,
            request.user.username
        )

        if cachekey in cache:
            return cache.get(cachekey)
    else:
        return False

    if name not in handlers._features:
        return False

    from .models import FeatureSwitch

    value = FeatureSwitch.objects.filter(
        user=request.user,
        feature=name
    ).exists()

    cache.set(cachekey, value)
    return value


def activate_feature(name, request):
    if not getattr(request, 'user', None):
        return False

    def a():
        u = request.user.is_authenticated
        return u() if callable(u) else u

    if not a():
        return False

    cachekey = 'feature.%s.activated.%s' % (
        name,
        request.user.username
    )

    if name not in handlers._features:
        return False

    from .models import FeatureSwitch

    if not FeatureSwitch.objects.filter(
        user=request.user,
        feature=name
    ).exists():
        FeatureSwitch.objects.create(
            user=request.user,
            feature=name
        )

        fa_signal.send(activate_feature, feature=name, request=request)
        cache.set(cachekey, True)

    return True


def deactivate_feature(name, request):
    if not getattr(request, 'user', None):
        return False

    def a():
        u = request.user.is_authenticated
        return u() if callable(u) else u

    if not a():
        return False

    cachekey = 'feature.%s.activated.%s' % (
        name,
        request.user.username
    )

    if name not in handlers._features:
        return False

    from .models import FeatureSwitch

    if FeatureSwitch.objects.filter(
        user=request.user,
        feature=name
    ).exists():
        FeatureSwitch.objects.filter(
            user=request.user,
            feature=name
        ).delete()

        fd_signal.send(deactivate_feature, feature=name, request=request)
        cache.set(cachekey, False)

    return True


def clear_feature_cache(request, *features):
    if not getattr(request, 'user', None):
        return False

    def a():
        u = request.user.is_authenticated
        return u() if callable(u) else u

    if not a():
        return

    for feature in handlers.list():
        if any(features) and feature not in features:
            continue

        cachekey = 'feature.%s.enabled.%s' % (
            feature,
            request.user.username
        )

        if cachekey in cache:
            cache.delete(cachekey)

        cachekey = 'feature.%s.activated.%s' % (
            feature,
            request.user.username
        )

        if cachekey in cache:
            cache.delete(cachekey)


def register(name, *groups):
    def wrapper(kls):
        if isinstance(kls, types.FunctionType):
            handlers.register(
                name,
                functional_handler_factory(kls),
                *groups
            )
        else:
            handlers.register(name, kls, *groups)

        return kls

    return wrapper



# Generated by Django 2.1.2 on 2019-01-08 14:33

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FeatureSwitch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('feature', models.CharField(max_length=100)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='activated_features', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'features_user',
            },
        ),
        migrations.AlterUniqueTogether(
            name='featureswitch',
            unique_together={('feature', 'user')},
        ),
    ]



from django.template import Library, TemplateSyntaxError, Node, NodeList
from django.utils.encoding import smart_str
from .. import feature_enabled, feature_activated

register = Library()


def parse_args_kwargs(parser, bits):
    args = []
    kwargs = {}
    bits = iter(bits)

    for bit in bits:
        for arg in bit.split(","):
            if '=' in arg:
                k, v = arg.split('=', 1)
                k = k.strip()
                kwargs[k] = parser.compile_filter(v)
            elif arg:
                args.append(parser.compile_filter(arg))

    return args, kwargs


def get_args_and_kwargs(args, kwargs, context):
    out_args = [arg.resolve(context) for arg in args]

    out_kwargs = dict(
        [
            (
                smart_str(k, 'ascii'),
                v.resolve(context)
            )
            for k, v in kwargs.items()
        ]
    )

    return out_args, out_kwargs


@register.tag()
def feature(parser, token):
    bits = token.contents.split(' ')

    if len(bits) < 1:
        raise TemplateSyntaxError(
            '%r tag requires at least one argument' % token.contents.split()[0]
        )

    nodelist_true = parser.parse(('else', 'endfeature'))
    token = parser.next_token()

    if token.contents == 'else':
        nodelist_false = parser.parse(('endfeature',))
        parser.delete_first_token()
    else:
        nodelist_false = NodeList()

    args, kwargs = parse_args_kwargs(parser, bits[1:])
    return FeatureNode(nodelist_true, nodelist_false, args, kwargs)


class FeatureNode(Node):
    def __init__(self, nodelist_true, nodelist_false, args, kwargs):
        if not any(args):
            raise TemplateSyntaxError(
                'feature tag expects at least one position argument'
            )

        if len(args) > 1:
            raise TemplateSyntaxError(
                'feature tag expects at most one position argument'
            )

        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.args = args
        self.kwargs = kwargs

    def render(self, context):
        features, kwargs = get_args_and_kwargs(self.args, self.kwargs, context)
        active = kwargs.pop('active', False)

        for key in kwargs.keys():
            raise TemplateSyntaxError(
                'feature tag got unexpected keyworwd argument: %s' % key
            )

        feature = features[0]
        request = context.get('request')

        if request is None:  # pragma: no cover
            return ''

        if feature_enabled(feature, request):
            if active and not feature_activated(feature, request):
                return self.nodelist_false.render(context)

            return self.nodelist_true.render(context)

        return self.nodelist_false.render(context)
